/* this is a UDP sender */

#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#define DATA_FRAME 1
#define ACK_FRAME 2
#define MAXBUFSIZE 10000

typedef struct 
{
  int size;
  int seq;
  int type;
  char buf[MAXBUFSIZE];
} Frame;


void HandleMessage(int sockfd,char *buf, int bufsize,  
		   struct sockaddr_in *servaddr, socklen_t socksize) 
{
  fd_set rset;
  struct timeval timeout;
  Frame myframe, ackframe;
  static int myseq = 0;
  
  myframe.size = bufsize;
  myframe.seq = myseq++;
  myframe.type = DATA_FRAME;
  memcpy(myframe.buf,buf,bufsize);

  sendto(sockfd, &myframe, sizeof(myframe)-MAXBUFSIZE+myframe.size, 0, 
	 (struct sockaddr *) servaddr, socksize);

  printf("\tSent %c\n", myframe.buf[0]);
  
  if(buf[0] == '+') /* To handle termination messages */
    return;
  
  FD_ZERO(&rset);
  FD_SET(sockfd, &rset);
  timeout.tv_sec = 1;
  timeout.tv_usec = 0;
      
  do {
    while (select(64, &rset, NULL, NULL, &timeout) == 0) 
    {/* timeout, retransmit */

      sendto(sockfd, &myframe, sizeof(myframe)-MAXBUFSIZE+myframe.size, 0, 
	     (struct sockaddr *) servaddr, socksize);

      printf("\tSent %c\n", myframe.buf[0]);

      FD_ZERO(&rset);
      FD_SET(sockfd, &rset);
      timeout.tv_sec = 1;
      timeout.tv_usec = 0;
    }

    recvfrom(sockfd, &ackframe, sizeof(ackframe), 0, NULL, NULL);

    if (ackframe.type != ACK_FRAME) 
    {
      fprintf(stderr, "ERROR: Client received a non-ACK_FRAME. Some thing is wrong ... exiting\n");
      exit(-1);
    }
    printf("Received ack %d\n", ackframe.seq);
  } while (ackframe.seq != myseq -1);

}


int main(int argc, char *argv[])
{
  int sockfd, i;
  struct sockaddr_in servaddr;
  char buf[MAXBUFSIZE];

  if (argc < 3) 
  {
    fprintf(stderr,"USAGE: %s ip_addr port.\n", argv[0]);
    exit(-1);
  }

  memset(&servaddr, 0, sizeof(servaddr));  
  servaddr.sin_family = AF_INET;
  inet_pton(AF_INET, argv[1], &servaddr.sin_addr);
  servaddr.sin_port = htons(atoi(argv[2]));

  if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) 
  {
    perror("socket");
    exit(-1);
  }

  buf[0] = 'A';
  for (i=0; i< 50; i++) 
  {
    HandleMessage(sockfd,buf,MAXBUFSIZE,&servaddr,sizeof(servaddr));

    buf[0]++;
    if (buf[0] == 'Z'+1) 
      buf[0] = 'A';
  }



  sleep(2);
  for (i=0; i<4; i++) 
  {
    buf[0] = '+';
    HandleMessage(sockfd,buf,1,&servaddr,sizeof(servaddr));
    sleep(1);
  }
  
  return 0; 
}












