
#include <sys/socket.h>
#include <sys/types.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>

#define DATA_FRAME 1
#define ACK_FRAME 2
#define MAXBUFSIZE 10000

typedef struct 
{
  int size;
  int seq;
  int type;
  char buf[MAXBUFSIZE];
} Frame;

int main(int argc, char *argv[])
{
  int sockfd, count, nextseq;
  socklen_t len;
  struct sockaddr_in servaddr;
  struct sockaddr_in cliaddr;
  FILE *fp;
  Frame myframe;
  
  if (argc < 2) 
  {
    fprintf(stderr,"USAGE: %s port.\n", argv[0]);
    exit(-1);
  }

  memset(&servaddr, 0, sizeof(servaddr));
  servaddr.sin_family = AF_INET;
  servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
  servaddr.sin_port = htons(atoi(argv[1]));

  if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) 
  {
    perror("socket");
    exit(-1);
  }

  if ((bind(sockfd, (struct sockaddr *)&servaddr, sizeof(servaddr)) < 0)) 
  {
    perror("bind");
    exit(-1);
  }

  if ((fp = fopen("outputfile", "w")) == NULL) 
  {
    printf("Can not open 'outputfile'\n");
    exit(-1);
  }

  count = nextseq = 0;
  while (1) 
  {
    /* sleep(1); */ /*   Just to slow down the server */
    memset(&cliaddr, 0, sizeof(servaddr));
    len = sizeof (cliaddr);
    recvfrom(sockfd, &myframe, sizeof(myframe), 0, (struct sockaddr *)&cliaddr, &len);

    printf("\t%d. Received %c\n", nextseq, myframe.buf[0]);

    if (!isupper(myframe.buf[0])) /* Terminate message */
      exit(0);

    /* Always send an acknowledgement */
    myframe.size = 0;
    myframe.type = ACK_FRAME;
    myframe.seq = myframe.seq; /* Just to show that the correct sequence number must be given */

#ifdef TEST
    if(count++ != 25) /* Simulate a lost acknowledgement */
      sendto(sockfd, &myframe, sizeof(myframe)-MAXBUFSIZE+myframe.size, 
	     0, (struct sockaddr *)&cliaddr, len);
    else
      sleep(2); 
#else
    sendto(sockfd, &myframe, sizeof(myframe)-MAXBUFSIZE+myframe.size, 
	   0, (struct sockaddr *)&cliaddr, len);
#endif 

    if (nextseq == myframe.seq) /* Should actually be converted to host byte order */
    {
      fprintf(fp, "%c\n", myframe.buf[0]);
      nextseq++;
    }
    
  }

  return 0;
}























